@@ -34,19 +34,26 @@
 class GApplication: public QApplication
 {
   Q_OBJECT
-  
+
 public:
-  GApplication(int argc, char **argv,
+  GApplication(int &argc, char **argv,
     struct GNUNET_GC_Configuration *cfg);
   void loadPlugins();
+  void setupMenuStruct();
   void showWindow();
   void setErrorContext(struct GNUNET_GE_Context *ectx);
   GMainWindow *getWindow();
-  
+
 protected:
+#if defined(Q_OS_WIN)
+  virtual bool winEventFilter(MSG *msg, long *result);
+#endif
+
   GMainWindow wnd;
   GPluginLoader loader;
   QString strCfgFile;
+
+  GMenuStruct menuStruct;
   struct GNUNET_GE_Context *ectx;
   struct GNUNET_GC_Configuration *cfg;
 };
