@@ -30,28 +30,37 @@
 #include <QWidget>
 #include <QMainWindow>
 #include <QLabel>
+#include <QSystemTrayIcon>
 
+#include "gnunet_qt_common.h"
 #include "ui_wndMain.h"
 #include "aboutDlg.h"
 
 class GMainWindow: public QMainWindow, protected Ui::WndMain
 {
   Q_OBJECT
-  
+
 public:
   GMainWindow(QWidget *parent = NULL);
   void addApplication(QWidget *wnd, const QIcon &icon, const QString &label);
+  virtual void setVisible (bool visible);
 
 public:
-  QLabel statusIcon, statusText, networkIcon, networkText;
+  QSystemTrayIcon *trayIcon;
+  QAction *action_Context_help, *actionAbout, *actionExit;
+
+  void loadMenuStruct(GMenuStruct &menuStruct);
 protected:
   virtual bool event(QEvent *event);
+
 public slots:
-  void setStatusText(const QPixmap &icon, const QString strText);
-  void setNetworkStatus(const QPixmap &icon, const QString strText);
+  void setStatusText(const QString &strIcon, const QString &strText);
+  void setNetworkStatus(const QString &strIcon, const QString &strText);
 protected slots:
   void contextHelp();
   void about();
+  void showAndHide();
+  void trayActivated(QSystemTrayIcon::ActivationReason reason);
 };
 
 #endif /*WNDMAIN_H_*/
