@@ -34,14 +34,14 @@
 class GAboutPlugin : public GPlugin, public Ui::WndAbout
 {
   Q_OBJECT
-  
+
 public:
   GAboutPlugin();
 
 // unused
 signals:
-    int setStatusText(const QPixmap &icon, const QString strStatus);
-    int setNetworkStatus(const QPixmap &icon, const QString strStatus);
+    int setStatusText(const QString &strIcon, const QString &strText);
+    int setNetworkStatus(const QString &strIcon, const QString &strText);
 
 protected:
   QString header();
