@@ -28,22 +28,23 @@
 #define DOWNLOADCONTROLLER_H_
 
 #include <QObject>
-#include <QStandardItemModel>
+#include "gnunet_qt_common.h"
 #include "fs.h"
 #include "ecrsMetaData.h"
 #include "downloadItemDelegate.h"
 
+
 class GFSDownloadController : public QObject
 {
   Q_OBJECT
-  
+
 public:
   typedef enum {COL_FILENAME, COL_SIZE, COL_PROGRESS, COL_STATUS, COL_ETA,
     COL_DST_PATH, COL_COUNT} GDownloadCols;
 
 	GFSDownloadController(GFSPlugin *fs);
 	virtual ~GFSDownloadController();
-  
+
   /**
    * @brief Initiates a download
    * @param parentSearch pointer to search
@@ -59,14 +60,14 @@ public:
     struct GNUNET_FSUI_SearchList *parentSearch, GFSEcrsUri &uri,
     GFSEcrsMetaData &meta, QString gnPath, QString name,
     QString destPath, int anonymity, bool recursive);
-    
+
   QPersistentModelIndex *started(struct GNUNET_FSUI_DownloadList *handle,
     QPersistentModelIndex *parent, const GNUNET_ECRS_FileInfo *fi, QString name,
     unsigned long long total, unsigned long long completed);
 
   void progress(QPersistentModelIndex *idx, unsigned long long completed,
     unsigned long long total, GNUNET_CronTime eta);
-    
+
   void completed(QPersistentModelIndex *idx, GFSEcrsUri uri, QString file);
 
   void state(QPersistentModelIndex *idx, GNUNET_FSUI_EventType type);
@@ -77,13 +78,19 @@ public:
 
   QAbstractItemModel *model();
 protected:
-  typedef QMap<GFSEcrsUri, QPersistentModelIndex> GFSDownloadList;
+  typedef struct
+  {
+    QPersistentModelIndex searchIdx;
+    bool suggestName;
+  } GDownloadInfo;
+
+  typedef QMap<GFSEcrsUri, GDownloadInfo> GFSDownloadList;
 
   GFSPlugin *fs;
-  QStandardItemModel downloadModel;
+  GItemModel downloadModel;
   GFSDownloadItemDelegate delegate;
   GFSDownloadList downloadList;
-  
+
   void setProgress(QPersistentModelIndex *idx, unsigned long long completed,
     unsigned long long total, GNUNET_CronTime eta);
 };
