@@ -32,12 +32,21 @@
 
 QSize GFSDownloadItemDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
 {
-  return QItemDelegate::sizeHint(option, index);
+  QSize ret;
+  
+  ((GItemModel *) index.model())->lock();
+  ret = QItemDelegate::sizeHint(option, index);
+  ((GItemModel *) index.model())->unlock();
+  
+  return ret;
 }
 
 void GFSDownloadItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
 {
-  int col = index.column();
+  int col;
+  
+  ((GItemModel *) index.model())->lock();
+  col = index.column();
   
   if (col == GFSDownloadController::COL_PROGRESS)
   {
@@ -57,6 +66,7 @@ void GFSDownloadItemDelegate::paint(QPainter *painter, const QStyleOptionViewIte
     
     qApp->style()->drawControl(QStyle::CE_ProgressBar, &barOptions, painter);
     
+    ((GItemModel *) index.model())->unlock();
     return;
   }
   else if (col == GFSDownloadController::COL_ETA)
@@ -75,6 +85,7 @@ void GFSDownloadItemDelegate::paint(QPainter *painter, const QStyleOptionViewIte
   }
   
   QItemDelegate::paint(painter, option, index);
+  ((GItemModel *) index.model())->unlock();
 }
 
 /* end of downloadItemDelegate.cc */
