@@ -20,7 +20,7 @@
 
 /**
  * @file src/plugins/fs/ecrsMetaData.cc
- * @brief Encapsulates GNUNET_ECRS_MetaData
+ * @brief Encapsulates GNUNET_MetaData
  * @author Nils Durner
  */
 
@@ -34,13 +34,13 @@ GFSEcrsMetaData::GFSEcrsMetaData()
 
 GFSEcrsMetaData::GFSEcrsMetaData(QByteArray serialized)
 {
-  metaData = GNUNET_ECRS_meta_data_deserialize(NULL, serialized.data(), serialized.size());
+  metaData = GNUNET_meta_data_deserialize(NULL, serialized.data(), serialized.size());
 }
 
 GFSEcrsMetaData::~GFSEcrsMetaData()
 {
   if (metaData)
-    GNUNET_ECRS_meta_data_destroy(metaData);
+    GNUNET_meta_data_destroy(metaData);
 }
 
 GFSEcrsMetaData::GFSEcrsMetaData(const GFSEcrsMetaData &src)
@@ -48,7 +48,7 @@ GFSEcrsMetaData::GFSEcrsMetaData(const GFSEcrsMetaData &src)
   copyFrom(src);
 }
 
-GFSEcrsMetaData::GFSEcrsMetaData(const struct GNUNET_ECRS_MetaData *src)
+GFSEcrsMetaData::GFSEcrsMetaData(const struct GNUNET_MetaData *src)
 {
   copyFrom(src);
 }
@@ -59,7 +59,7 @@ GFSEcrsMetaData &GFSEcrsMetaData::operator=(const class GFSEcrsMetaData &src)
   return *this;
 }
 
-GFSEcrsMetaData &GFSEcrsMetaData::operator=(const struct GNUNET_ECRS_MetaData *src)
+GFSEcrsMetaData &GFSEcrsMetaData::operator=(const struct GNUNET_MetaData *src)
 {
   copyFrom(src);
   return *this;
@@ -67,15 +67,15 @@ GFSEcrsMetaData &GFSEcrsMetaData::operator=(const struct GNUNET_ECRS_MetaData *s
   
 void GFSEcrsMetaData::copyFrom(const class GFSEcrsMetaData &src)
 {
-  metaData = GNUNET_ECRS_meta_data_duplicate(src.metaData);
+  metaData = GNUNET_meta_data_duplicate(src.metaData);
 }
 
-void GFSEcrsMetaData::copyFrom(const struct GNUNET_ECRS_MetaData *src)
+void GFSEcrsMetaData::copyFrom(const struct GNUNET_MetaData *src)
 {
-  metaData = GNUNET_ECRS_meta_data_duplicate(src);
+  metaData = GNUNET_meta_data_duplicate(src);
 }
 
-struct GNUNET_ECRS_MetaData *GFSEcrsMetaData::meta()
+struct GNUNET_MetaData *GFSEcrsMetaData::meta()
 {
   return metaData;
 }
@@ -84,15 +84,15 @@ QByteArray GFSEcrsMetaData::serialized()
 {
   int size;
   
-  size = GNUNET_ECRS_meta_data_get_serialized_size(metaData, GNUNET_ECRS_SERIALIZE_FULL);
+  size = GNUNET_meta_data_get_serialized_size(metaData, GNUNET_SERIALIZE_FULL);
   if (size != GNUNET_SYSERR)
   {
     QByteArray ret;
     char *data;
 
     data = new char[size];
-    GNUNETQT_ASSERT(GNUNET_ECRS_meta_data_serialize(NULL, metaData, data, size,
-      GNUNET_ECRS_SERIALIZE_FULL) != GNUNET_SYSERR);
+    GNUNETQT_ASSERT(GNUNET_meta_data_serialize(NULL, metaData, data, size,
+      GNUNET_SERIALIZE_FULL) != GNUNET_SYSERR);
     
     ret = QByteArray(data, size);
     delete [] data;
