@@ -20,7 +20,7 @@
 
 /**
  * @file src/plugins/fs/ecrsMetaData.h
- * @brief Encapsulates GNUNET_ECRS_MetaData
+ * @brief Encapsulates GNUNET_MetaData
  * @author Nils Durner
  */
 
@@ -36,21 +36,21 @@ public:
 	GFSEcrsMetaData();
   GFSEcrsMetaData(QByteArray serialized);
   GFSEcrsMetaData(const class GFSEcrsMetaData &src);
-  GFSEcrsMetaData(const struct GNUNET_ECRS_MetaData *src);
+  GFSEcrsMetaData(const struct GNUNET_MetaData *src);
   virtual ~GFSEcrsMetaData();
   
   GFSEcrsMetaData &operator=(const class GFSEcrsMetaData &src);
-  GFSEcrsMetaData &operator=(const struct GNUNET_ECRS_MetaData *src);
+  GFSEcrsMetaData &operator=(const struct GNUNET_MetaData *src);
   
   QByteArray serialized();
   
-  struct GNUNET_ECRS_MetaData *meta();
+  struct GNUNET_MetaData *meta();
   
 protected:
-  struct GNUNET_ECRS_MetaData *metaData;
+  struct GNUNET_MetaData *metaData;
 
   void copyFrom(const class GFSEcrsMetaData &src);
-  void copyFrom(const struct GNUNET_ECRS_MetaData *src);
+  void copyFrom(const struct GNUNET_MetaData *src);
 };
 
 #endif /*ECRSMETADATA_H_*/
