@@ -26,6 +26,7 @@
 
 #include <QHeaderView>
 #include <QMenu>
+#include <QClipboard>
 #include <extractor.h>
 
 #include "gnunet_qt_common.h"
@@ -35,19 +36,21 @@
 GFSSearch::GFSSearch(class GFSPlugin *fs, QTabWidget *tab) : QWidget()
 {
   QHeaderView *header;
-  
+
   setupUi(this);
 
   header = treeResults->header();
   header->setContextMenuPolicy(Qt::CustomContextMenu);
   treeResults->setUniformRowHeights(false);
+  treeResults->addAction(actionCopy_URI);
   treeResults->addAction(actionDownload);
   treeResults->setEditTriggers(QAbstractItemView::NoEditTriggers);
   this->tab = tab;
   this->fs = fs;
-  
+
   connect(pbClose, SIGNAL(clicked(bool)), this, SLOT(closeClicked()));
   connect(pbDownload, SIGNAL(clicked(bool)), this, SLOT(downloadClicked()));
+  connect(actionCopy_URI, SIGNAL(triggered()), this, SLOT(copyURI()));
   connect(actionDownload, SIGNAL(triggered()), this, SLOT(downloadClicked()));
   connect(header, SIGNAL(customContextMenuRequested(const QPoint &)), this,
     SLOT(headerRightClicked(const QPoint &)));
@@ -62,35 +65,75 @@ void GFSSearch::closeClicked()
 
 void GFSSearch::downloadClicked()
 {
-  QModelIndexList allSel, uniqSel;
+  QModelIndexList allSel;
+  GPersistentModelIndexList uniqSel;
   QModelIndexList::iterator it;
-  
+
   /* Every column is treated as separate selection */
   allSel = treeResults->selectionModel()->selectedIndexes();
   for (it = allSel.begin(); it != allSel.end(); it++)
   {
     QModelIndex &idx = *it;
-    
+
     if (idx.column() == 0)
       uniqSel.append(idx);
   }
-  
+
   emit download(m, uri, uniqSel, spinAnon->value(), cbRecursive->isChecked());
 }
 
+void GFSSearch::copyURI()
+{
+  QModelIndexList allSel;
+  GPersistentModelIndexList uniqSel;
+  QModelIndexList::iterator it;
+  QString strUris;
+
+  /* Every column is treated as separate selection */
+  allSel = treeResults->selectionModel()->selectedIndexes();
+  for (it = allSel.begin(); it != allSel.end(); it++)
+  {
+    QModelIndex &idx = *it;
+
+    if (idx.column() == MODEL_IDX(SEARCH_URI_COLUMN))
+    {
+      if (strUris != "")
+#if defined(Q_WS_WIN)
+        strUris += "\r\n";
+#else
+        strUris += "\n";
+#endif
+      strUris += idx.model()->data(idx).toString();
+    }
+  }
+
+  QApplication::clipboard()->setText(strUris);
+}
+
 void GFSSearch::resultExpanded(const QModelIndex &index)
 {
   QModelIndex idx;
-  
-  idx = index.model()->index(index.row(), EXTRACTOR_MIMETYPE, index.parent());
-  if (idx.model()->data(idx) == "application/gnunet-directory")
+  GItemModel *model;
+
+  model = (GItemModel *) index.model();
+  model->lock();
+  if (model->data(index, Qt::UserRole) != RESULT_DOWNLOADED)
   {
-    QModelIndexList list;
-    
-    list.append(idx);
-    
-    emit download(m, uri, list, spinAnon->value(), false); 
+    idx = model->index(index.row(), EXTRACTOR_MIMETYPE, index.parent());
+    if (model->data(idx) == "application/gnunet-directory")
+    {
+      GPersistentModelIndexList list;
+
+      list.append(index);
+
+      model->unlock();
+      emit download(m, uri, list, spinAnon->value(), false);
+
+      return;
+    }
   }
+
+  model->unlock();
 }
 
 void GFSSearch::clear()
@@ -112,7 +155,7 @@ void GFSSearch::addColumn(QMenu *menu, QString strTitle, int nr)
   action = menu->addAction(strTitle);
   action->setCheckable(true);
   action->setData(QVariant(nr));
-  
+
   if (!treeResults->header()->isSectionHidden(MODEL_IDX(nr)))
     action->setChecked(true);
 }
@@ -121,35 +164,35 @@ void GFSSearch::headerRightClicked(const QPoint &pos)
 {
   EXTRACTOR_KeywordType typeItem;
   QAction *item;
-  
+
   QMenu *menu = new QMenu(treeResults->header());
   menu->move(pos);
-  
+
   typeItem = EXTRACTOR_getHighestKeywordTypeNumber();
   while(typeItem >= (EXTRACTOR_KeywordType) 0)
   {
     QString strItem = metaTypeName(typeItem);
-    
+
     if (strItem != "")
       addColumn(menu, strItem, typeItem);
-    
+
     typeItem = (EXTRACTOR_KeywordType) ((int) typeItem - 1);
   }
-  
+
   // add availability and relevance
   addColumn(menu, tr("Ranking"), SEARCH_RANK_COLUMN);
-  
+
   item = menu->exec();
   if (item)
   {
     setColHidden(item->data().toInt(), !item->isChecked());
-    
+
     // kludge: hiding rows causes recalculation of row heights.
     // This is necessary because we calculate the height for visible columns only.
     treeResults->setRowHidden(0, QModelIndex(), true);
     treeResults->setRowHidden(0, QModelIndex(), false);
   }
-  
+
   delete menu;
 }
 
@@ -165,20 +208,20 @@ void GFSSearch::setupColumns()
   GIntList cols;
   EXTRACTOR_KeywordType maxType;
   int typeIdx;
-  
+
   maxType = EXTRACTOR_getHighestKeywordTypeNumber();
   m->setColumnCount(SEARCH_MAX_COLUMN + 1);
   for (typeIdx = 0; typeIdx <= maxType; typeIdx++)
   {
     GString label;
-    
+
     label = metaTypeName(MODEL_IDX(typeIdx));
     label.proper();
-    
+
     m->setHeaderData(typeIdx, Qt::Horizontal, label, Qt::DisplayRole);
   }
   m->setHeaderData(SEARCH_RANK_COLUMN, Qt::Horizontal, tr("Ranking"), Qt::DisplayRole);
-    
+
   // get columns to be displayed
   // FIXME
   cols.append(EXTRACTOR_FILENAME);
@@ -186,7 +229,7 @@ void GFSSearch::setupColumns()
   cols.append(EXTRACTOR_MIMETYPE);
   cols.append(EXTRACTOR_THUMBNAIL_DATA);
   cols.append(SEARCH_RANK_COLUMN);
-  
+
   for (colIdx = 0; colIdx < m->columnCount(); colIdx++)
     setColHidden(MODEL_IDX(colIdx), !cols.contains(colIdx));
   treeResults->setColumnWidth(0, 200);
@@ -195,17 +238,17 @@ void GFSSearch::setupColumns()
   treeResults->setColumnWidth(SEARCH_RANK_COLUMN, 50);
 }
 
-void GFSSearch::setModel(QStandardItemModel *model)
+void GFSSearch::setModel(GItemModel *model)
 {
   QByteArray data;
-  
+
   m = model;
-  treeResults->setModel(model);
+  treeResults->setModel(model->abstractItemModel());
   connect(model, SIGNAL(rowsInserted(const QModelIndex &, int, int)), this,
     SLOT(resultInserted()));
 
   setupColumns();
-    
+
   resultInserted();
 }
 
