@@ -28,9 +28,9 @@
 #define SEARCHRESULTS_H_
 
 #include <QTabWidget>
-#include <QStandardItemModel>
 #include <GNUnet/gnunet_fsui_lib.h>
 
+#include "gnunet_qt_common.h"
 #include "fs.h"
 #include "ecrsuri.h"
 #include "ui_fs-search-result.h"
@@ -38,32 +38,33 @@
 class GFSSearch : public QWidget, protected Ui::ResultWnd
 {
   Q_OBJECT
-  
+
 public:
   GFSSearch(class GFSPlugin *fs, QTabWidget *tab);
   void clear();
-  void setModel(QStandardItemModel *model);
+  void setModel(GItemModel *model);
   void setItemDelegate(QAbstractItemDelegate *itemDelegate);
   void setUri(GFSEcrsUri &uri);
-  QStandardItemModel *model();
-  
+  GItemModel *model();
+
 signals:
   void closeSearchWnd(GFSEcrsUri &uri);
-  void download(QStandardItemModel *model, GFSEcrsUri &uri,
-    QModelIndexList indexes, int anonymity, bool recursive);
+  void download(GItemModel *model, GFSEcrsUri &uri,
+      GPersistentModelIndexList indexes, int anonymity, bool recursive);
 
 protected:
   class GFSPlugin *fs;
   GFSEcrsUri uri;
   QTabWidget *tab;
-  QStandardItemModel *m;
-  
+  GItemModel *m;
+
   void setColHidden(int col, bool hidden);
   void setupColumns();
   void addColumn(QMenu *menu, QString strTitle, int nr);
 protected slots:
   void closeClicked();
   void downloadClicked();
+  void copyURI();
   void headerRightClicked(const QPoint &pos);
   void resultInserted();
   void resultExpanded(const QModelIndex &index);
