@@ -25,22 +25,22 @@
  */
 
 #include <QEvent>
+#include <QMenu>
 #include <QLineEdit>
 #include <QMessageBox>
 #include <QFileDialog>
 #include <QDir>
 #include <QUrl>
-#include <QStandardItemModel>
 #include <QDesktopServices>
+#include <QClipboard>
 #include <GNUnet/gnunet_fsui_lib.h>
-#if defined(Q_WS_WIN)
-  #include <windows.h>
-#endif
+
 
 #include "gnunet_qt_common.h"
 #include "fs.h"
 #include "fs-search.h"
 #include "ecrsuri.h"
+#include "fs-open-uri.h"
 #include "searchController.h"
 #include "downloadController.h"
 
@@ -48,7 +48,7 @@ static void *fsuiEventProcessor(void *cls, const GNUNET_FSUI_Event *event)
 {
   void *ret;
   int i;
-  
+
   GFSSearchController *searchCntrl = ((GFSPlugin *) cls)->searchController();
   GFSUploadController *uploadCntrl = ((GFSPlugin *) cls)->uploadController();
   GFSDownloadController *downloadCntrl = ((GFSPlugin *) cls)->downloadController();
@@ -93,20 +93,24 @@ static void *fsuiEventProcessor(void *cls, const GNUNET_FSUI_Event *event)
       break;
     case GNUNET_FSUI_upload_started:
       return uploadCntrl->started((QPersistentModelIndex *) event->data.UploadStarted.uc.pcctx,
-        event->data.UploadStarted.filename, event->data.UploadStarted.total, 0);
+        event);
     case GNUNET_FSUI_upload_resumed:
-      return uploadCntrl->started((QPersistentModelIndex *) event->data.UploadResumed.uc.pcctx,
-        event->data.UploadResumed.filename, event->data.UploadResumed.total,
-        event->data.UploadResumed.completed);
+      return uploadCntrl->resumed((QPersistentModelIndex *) event->data.UploadStarted.uc.pcctx,
+        event);
     case GNUNET_FSUI_upload_progress:
       uploadCntrl->progress((QPersistentModelIndex *) event->data.UploadProgress.uc.cctx,
         event->data.UploadProgress.completed, event->data.UploadProgress.total);
+      uploadCntrl->state((QPersistentModelIndex *) event->data.UploadProgress.uc.cctx,
+          event->type);
+      break;
+    case GNUNET_FSUI_upload_completed:
+      uploadCntrl->complete((QPersistentModelIndex *) event->data.UploadStopped.uc.cctx,
+          event->data.UploadCompleted.uri);
       break;
     case GNUNET_FSUI_upload_error:
       GNUNET_GE_LOG(errorContext, (GNUNET_GE_KIND) (GNUNET_GE_ERROR | GNUNET_GE_USER | GNUNET_GE_IMMEDIATE),
         event->data.UploadError.message);
     case GNUNET_FSUI_upload_stopped:
-    case GNUNET_FSUI_upload_completed:
     case GNUNET_FSUI_upload_aborted:
     case GNUNET_FSUI_upload_suspended:
       uploadCntrl->state((QPersistentModelIndex *) event->data.UploadStopped.uc.cctx,
@@ -144,35 +148,52 @@ static void *fsuiEventProcessor(void *cls, const GNUNET_FSUI_Event *event)
       break;
     default:
       GNUNET_GE_LOG(errorContext, (GNUNET_GE_KIND) (GNUNET_GE_ERROR | GNUNET_GE_USER | GNUNET_GE_IMMEDIATE),
-        qPrintable(QObject::tr("Internal error: received unhandled event from FSUI.")));      
+        qPrintable(QObject::tr("Internal error: received unhandled event from FSUI.")));
   }
 
   return NULL;
 }
 
-GFSPlugin::GFSPlugin(struct GNUNET_GC_Configuration *config,
-  struct GNUNET_GE_Context *errorContext) : GPlugin()
+GFSPlugin::GFSPlugin(GPluginInitParams *params) : GPlugin()
 {
   setupUi(this);
 
   qRegisterMetaType<GFSEcrsUri>("GFSEcrsUri&");
-  qRegisterMetaType<QStandardItemModel *>("QStandardItemModel*");
+  qRegisterMetaType<GFSEcrsUri>("GFSEcrsUri");
+  qRegisterMetaType<GItemModel *>("GItemModel*");
   qRegisterMetaType<QModelIndex>("QModelIndex");
   qRegisterMetaType<Qt::Orientation>("Qt::Orientation");
   qRegisterMetaType<QModelIndexList>("QModelIndexList");
+  qRegisterMetaType<GPersistentModelIndexList>("GPersistentModelIndexList");
   qRegisterMetaType<GRanking>("GRanking");
 
-  cfg = config;
-  ectx = errorContext;
-  
+  cfg = params->config;
+  ectx = params->errorContext;
+
+  // Menu
+  GMenuAction action;
+  GMenuStruct::iterator itMenu;
+
+  action.name = "gnunet-qt::fs::openURI";
+  action.text = tr("Open &URI");
+  action.receiver = this;
+  action.receiverSlot = SLOT(openURI());
+  action.action = &actionOpenURI;
+
+  for (itMenu = params->menu->begin(); itMenu != params->menu->end(); itMenu++)
+    if (itMenu->name == "gnunet-qt::core::file")
+      itMenu->actions.push_front(action);
+
   // FS tab
   connect(pbSearch, SIGNAL(clicked(bool)), this, SLOT(searchClicked()));
   connect(cmbSearchFor->lineEdit(), SIGNAL(returnPressed()), this, SLOT(searchClicked()));
   searchItemDelegate = new GSearchItemDelegate();
-  
+
   treeDownloads->addAction(actionOpen_download);
+  treeUploads->addAction(actionCopy_URI);
   connect(actionOpen_download, SIGNAL(triggered()), this, SLOT(openDownloadClicked()));
-  
+  connect(actionCopy_URI, SIGNAL(triggered()), this, SLOT(copyUploadURIClicked()));
+
   tabResults->removeTab(0); // created by Qt Designer
   GFSSearch *view = new GFSSearch(this, tabResults);
   view->setEnabled(false);
@@ -190,12 +211,12 @@ GFSPlugin::GFSPlugin(struct GNUNET_GC_Configuration *config,
 
   // Download
   downloadCntrl = new GFSDownloadController(this);
-  
+
   // Summary
   connect(pbClearDL, SIGNAL(clicked(bool)), this, SLOT(clearDLClicked()));
   connect(pbCancelDL, SIGNAL(clicked(bool)), this, SLOT(cancelDLClicked()));
   connect(pbClearUL, SIGNAL(clicked(bool)), this, SLOT(clearULClicked()));
-  
+
   treeSearches->setColumnWidth(0, 115);
   treeSearches->setColumnWidth(1, 70);
   treeSearches->setColumnWidth(2, 70);
@@ -209,7 +230,7 @@ GFSPlugin::GFSPlugin(struct GNUNET_GC_Configuration *config,
   treeUploads->setEditTriggers(QAbstractItemView::NoEditTriggers);
 
   /* FIXME: allow user to configure download parallelism */
-  fsuiContext = GNUNET_FSUI_start(errorContext, config, "gnunet_qt", 128, GNUNET_YES,
+  fsuiContext = GNUNET_FSUI_start(params->errorContext, params->config, "gnunet_qt", 128, GNUNET_YES,
     fsuiEventProcessor, this);
 }
 
@@ -222,16 +243,40 @@ GFSPlugin::~GFSPlugin()
   delete downloadCntrl;
 }
 
+void GFSPlugin::openURI()
+{
+  GFSOpenURIDialog *dialog;
+  QPersistentModelIndex idx;
+  QString uri;
+  GFSEcrsUri ecrsUri;
+  GFSEcrsMetaData md;
+
+  dialog = new GFSOpenURIDialog(this);
+  if (dialog->exec() == QDialog::Accepted)
+  {
+    QString strIcon;
+
+    uri = dialog->uriInput->toPlainText();
+    ecrsUri = uri;
+    downloadCntrl->start(idx, NULL, ecrsUri,
+        md, "", dialog->file->text(), "", dialog->anon->value(),
+        dialog->recursive->isChecked());
+
+    strIcon = ":/pixmaps/download.png";
+    emit setStatusText(strIcon, tr("Download of \"%0\" queued.").arg(ecrsUri.toString().left(16)));
+  }
+}
+
 void GFSPlugin::openDownloadClicked()
 {
   QModelIndexList sel;
   QModelIndexList::iterator it;
-  
+
   sel = treeDownloads->selectionModel()->selectedIndexes();
   for (it = sel.begin(); it != sel.end(); it++)
   {
     QModelIndex idx = *it;
-    
+
     if (idx.column() == GFSDownloadController::COL_DST_PATH)
     {
       QString name;
@@ -240,22 +285,40 @@ void GFSPlugin::openDownloadClicked()
       QFileInfo info(name);
 
       // Open file
-#if defined(Q_WS_WIN)
-      // Qt 4.2 URL encodes files which is not understood by Windows, so
-      // we call ShellExecute directly instead of openUrl()
-      ShellExecuteA(0, 0, qPrintable(name), 0, 0, SW_SHOWNORMAL);
-#else
-      QUrl url;
-      
-      url.setScheme("file");
-      url.setPath(info.absoluteFilePath());
-      
-      QDesktopServices::openUrl(url);
-#endif
+      GDesktopServices::openDocument(qPrintable(info.absoluteFilePath()));
     }
   }
 }
 
+void GFSPlugin::copyUploadURIClicked()
+{
+  QModelIndexList sel;
+  QModelIndexList::iterator it;
+  QString strUris;
+
+  sel = treeUploads->selectionModel()->selectedIndexes();
+  for (it = sel.begin(); it != sel.end(); it++)
+  {
+    QModelIndex idx = *it;
+
+    if (idx.column() == GFSUploadController::COL_URI)
+    {
+      GFSEcrsUri uri;
+
+      uri = uploadCntrl->model()->data(idx).value<GFSEcrsUri>();
+      if (strUris != "")
+#if defined(Q_WS_WIN)
+        strUris += "\r\n";
+#else
+        strUris += "\n";
+#endif
+      strUris += uri.serialized();
+    }
+  }
+
+  QApplication::clipboard()->setText(strUris);
+}
+
 void GFSPlugin::searchClicked()
 {
   QString strSearch, strNS;
@@ -271,7 +334,7 @@ void GFSPlugin::searchClicked()
 
   if (cmbSearchFor->findText(strSearch) == -1)
     cmbSearchFor->addItem(strSearch);
-    
+
   strNS = cmbNS->lineEdit()->text();
 
   /* Create URI */
@@ -279,7 +342,7 @@ void GFSPlugin::searchClicked()
   {
     QString strUri;
     char *ustring;
-    
+
     strUri = QString(GNUNET_ECRS_URI_PREFIX) + GNUNET_ECRS_SUBSPACE_INFIX + strNS + "/" +
       strSearch;
     ustring = strUri.toLocal8Bit().data();
@@ -294,9 +357,9 @@ void GFSPlugin::searchClicked()
 
   if (uri == NULL || searchCntrl->isActive(uri))
     return;
-  
+
   /* Start search */
-  GNUNET_FSUI_search_start(fsuiContext, spinSearchAnon->value(), 
+  GNUNET_FSUI_search_start(fsuiContext, spinSearchAnon->value(),
     uri);
 
   GNUNET_ECRS_uri_destroy(uri);
@@ -317,12 +380,12 @@ GFSDownloadController *GFSPlugin::downloadController()
   return downloadCntrl;
 }
 
-QAbstractItemView *GFSPlugin::searchSummaryView()
+QTreeView *GFSPlugin::searchSummaryView()
 {
   return treeSearches;
 }
 
-QAbstractItemView *GFSPlugin::uploadView()
+QTreeView *GFSPlugin::uploadView()
 {
   return treeUploads;
 }
@@ -335,23 +398,23 @@ QTreeView *GFSPlugin::downloadView()
 bool GFSPlugin::event(QEvent *e)
 {
   EventType type;
-  
+
   GPlugin::event(e);
-  
+
   GEvent *event = dynamic_cast<class GEvent *> (e);
-  
+
   if (!event)
     return false;
-    
+
   type = (EventType) e->type();
-  
+
   if (type == NewSearch)
   {
     /* Setup new search window */
     GFSSearch *tab;
     int tabIdx;
     GFSNewSearchInfo *info = (GFSNewSearchInfo *) event->getParam();
-    
+
     /* The first tab is always there. Use this one if it isn't used (disabled) yet. */
     tab = (GFSSearch *) tabResults->widget(0);
     if (tab->isEnabled())
@@ -367,14 +430,14 @@ bool GFSPlugin::event(QEvent *e)
       tab->setEnabled(true);
       tabIdx = 0;
     }
-    
+
     tab->setItemDelegate(searchItemDelegate);
     tabResults->setCurrentIndex(tabIdx);
 
     tab->setUri(info->uri);
     tab->setModel(info->model);
     event->setReturn(tab);
-    
+
     return true;
   }
   else if (type == CloseSearch)
@@ -396,20 +459,20 @@ bool GFSPlugin::event(QEvent *e)
       tabResults->setTabText(0, tr("Search results"));
       lastTab = true;
     }
-    
+
     event->setReturn((void *) lastTab); // wakeup posting thread
   }
-  
+
   return false;
 }
 
 void GFSPlugin::chooseClicked()
 {
   QString strFile;
-    
+
   if (rbFile->isChecked())
     strFile = QFileDialog::getOpenFileName(this,
-      tr("File to publish"), editName->text());    
+      tr("File to publish"), editName->text());
   else
     strFile = QFileDialog::getExistingDirectory(this,
       tr("Choose a directory to publish:"), editName->text()).replace("\"", "\\\"");
@@ -420,22 +483,22 @@ void GFSPlugin::chooseClicked()
 void GFSPlugin::uploadClicked()
 {
   QString strPath;
-  QPixmap icon;
+  QString strIcon;
 
   strPath = editName->text();
   if (strPath == "")
   {
     QMessageBox::critical(this, tr("Error"), tr("No filename specified"),
       QMessageBox::Ok, QMessageBox::NoButton, QMessageBox::NoButton);
-      
+
     return;
   }
-  
+
   if (uploadCntrl->start(this, strPath, rbIndex->isChecked(),
     spinPrio->value(), spinUploadAnon->value()))
   {
-    icon.load(":/pixmaps/upload.png");
-    emit setStatusText(icon, tr("Publication of \"%0\" queued.").arg(strPath));
+    strIcon = ":/pixmaps/upload.png";
+    emit setStatusText(strIcon, tr("Publication of \"%0\" queued.").arg(strPath));
   }
 }
 
@@ -443,13 +506,13 @@ void GFSPlugin::download(QPersistentModelIndex &searchIdx,
   struct GNUNET_FSUI_SearchList *handle, GFSEcrsUri &uri, GFSEcrsMetaData &meta,
   QString gnPath, QString &file, int anonymity, bool recurse)
 {
-  QPixmap icon;
+  QString strIcon;
 
   downloadCntrl->start(searchIdx, handle, uri, meta, gnPath, file, "",
     anonymity, recurse);
 
-  icon.load(":/pixmaps/download.png");
-  emit setStatusText(icon, tr("Download of \"%0\" queued.").arg(file));
+  strIcon = ":/pixmaps/download.png";
+  emit setStatusText(strIcon, tr("Download of \"%0\" queued.").arg(file));
 }
 
 void GFSPlugin::clearDLClicked()
@@ -460,20 +523,20 @@ void GFSPlugin::clearDLClicked()
 void GFSPlugin::cancelDLClicked()
 {
   typedef QList<struct GNUNET_FSUI_DownloadList *> GCancelList;
-  
+
   QModelIndexList list;
   QModelIndexList::iterator it;
   GCancelList cancelList;
   GCancelList::iterator itCancel;
   QItemSelectionModel *selModel = treeDownloads->selectionModel();
   QAbstractItemModel *dataModel = treeDownloads->model();
-  
+
   list = selModel->selectedRows();
-  
+
   // collect handles of downloads first because indexes shift as rows are removed
   for (it = list.begin(); it != list.end(); it++)
     cancelList.push_back((struct GNUNET_FSUI_DownloadList *) dataModel->data(*it, Qt::UserRole).value<void *>());
-  
+
   // remove downloads
   for (itCancel = cancelList.begin(); itCancel != cancelList.end(); itCancel++)
     downloadCntrl->cancel(*itCancel);
@@ -508,7 +571,7 @@ QString GFSPlugin::fsuiState(GNUNET_FSUI_EventType type)
     case GNUNET_FSUI_upload_started:
     case GNUNET_FSUI_unindex_started:
       return tr("started");
-      
+
     case GNUNET_FSUI_search_result:
     case GNUNET_FSUI_download_progress:
     case GNUNET_FSUI_upload_progress:
@@ -548,7 +611,7 @@ QString GFSPlugin::fsuiState(GNUNET_FSUI_EventType type)
     case GNUNET_FSUI_upload_resumed:
     case GNUNET_FSUI_unindex_resumed:
       return tr("resumed");
-      
+
     default:
       return QString();
   }
@@ -556,12 +619,11 @@ QString GFSPlugin::fsuiState(GNUNET_FSUI_EventType type)
 
 extern "C"
 {
-  GNUNETQT_API GPlugin *init_fs(struct GNUNET_GC_Configuration *config,
-    struct GNUNET_GE_Context *errorContext)
+  GNUNETQT_API GPlugin *init_fs(GPluginInitParams *params)
   {
-    return new GFSPlugin(config, errorContext);
+    return new GFSPlugin(params);
   }
-  
+
   GNUNETQT_API void shutdown_fs(GPlugin *plugin)
   {
     delete plugin;
