@@ -35,31 +35,48 @@
 
 QSize GSearchItemDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
 {
-  QVariant var = index.model()->headerData(index.column(), Qt::Horizontal, Qt::UserRole);
+  GItemModel *model;
+  QVariant var;
+  QSize ret;
+  
+  model = (GItemModel *) index.model();
+  model->lock();
+  var = model->headerData(index.column(), Qt::Horizontal, Qt::UserRole);
   
   if (var.toInt() == 1)
+  {
+    model->unlock();
     return QSize(0, 0);
+  }
   
-  var = index.model()->data(index);
+  var = model->data(index);
   if (index.column() == MODEL_IDX(EXTRACTOR_THUMBNAIL_DATA) && var.type() == QVariant::ByteArray)
   {
     QImage img;
     
+    model->unlock();
     img.loadFromData(var.toByteArray());
     
     return img.size();
   }
 
-  return QItemDelegate::sizeHint(option, index);
+  ret = QItemDelegate::sizeHint(option, index);
+  model->unlock();
+  return ret;
 }
 
 void GSearchItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
 {
   QVariant var;
+  GItemModel *model;
+  int col;
   
-  var = index.model()->data(index);
+  model = (GItemModel *) index.model();
+  model->lock();
+  var = model->data(index);
+  col = index.column();
   
-  if (index.column() == MODEL_IDX(EXTRACTOR_THUMBNAIL_DATA) && var.type() == QVariant::ByteArray)
+  if (col == MODEL_IDX(EXTRACTOR_THUMBNAIL_DATA) && var.type() == QVariant::ByteArray)
   {
     QImage img;
     int left;
@@ -75,7 +92,7 @@ void GSearchItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &o
     
     painter->drawImage(QPoint(left, option.rect.top()), img);
   }
-  else if (index.column() == SEARCH_RANK_COLUMN)
+  else if (col == SEARCH_RANK_COLUMN)
   {
     GRanking rank;
     QString str;
@@ -123,13 +140,15 @@ void GSearchItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &o
     painter->drawLine(rect.x() + colwidth / 2, rect.y(),
         rect.x() + colwidth / 2, rect.y() + colheight / 2 - 1);
   }
-  else if (index.model()->data(index, Qt::UserRole) == 1)
+  else if (model->data(index, Qt::UserRole) == 1)
   {
     // expanded directory
     painter->drawText(option.rect, tr("Retrieving content..."));
   }
   else
     QItemDelegate::paint(painter, option, index);
+  
+  model->unlock();
 }
 
 /* end of searchItemDelegate.cc */
