@@ -28,7 +28,7 @@
 
 GFSSearchSummaryController::GFSSearchSummaryController(class GFSPlugin *fs)
 {
-  model = new GFSSearchSummaryModel();
+  model = new GFSSearchSummaryModel(fs->config(), fs->errorContext());
   this->fs = fs;
   fs->searchSummaryView()->setModel(model);
 }
