@@ -46,8 +46,11 @@ QVariant GFSSearchSummaryModel::headerData(int section, Qt::Orientation orientat
   return QVariant();
 }
 
-GFSSearchSummaryModel::GFSSearchSummaryModel() : QAbstractItemModel()
+GFSSearchSummaryModel::GFSSearchSummaryModel(struct GNUNET_GC_Configuration *cfg,
+    struct GNUNET_GE_Context *ectx) : QAbstractItemModel()
 {
+  this->cfg = cfg;
+  this->ectx = ectx;
 }
 
 QModelIndex GFSSearchSummaryModel::index(int row, int column,
@@ -91,7 +94,7 @@ QVariant GFSSearchSummaryModel::data(const QModelIndex &index, int role) const
     switch(index.column())
     {
       case 0:
-        return QVariant(result.uri.toString());
+        return QVariant(result.uri.toDisplayString(cfg, ectx));
       case 1:
         return QVariant(result.count);
       case 2:
