@@ -39,7 +39,8 @@ class GFSSearchSummaryModel : public QAbstractItemModel
   Q_OBJECT
   
 public:
-  GFSSearchSummaryModel();
+  GFSSearchSummaryModel(struct GNUNET_GC_Configuration *cfg,
+      struct GNUNET_GE_Context *ectx);
   QVariant headerData(int section, Qt::Orientation orientation,
     int role = Qt::DisplayRole) const;
   QModelIndex index(int row, int column, const QModelIndex &parent =
@@ -72,6 +73,8 @@ protected:
   GFSSearchList::iterator find(const struct GNUNET_FSUI_SearchList *handle); 
 
   GFSSearchList results;
+  struct GNUNET_GC_Configuration *cfg;
+  struct GNUNET_GE_Context *ectx;
 };
 
 #endif /*SEARCHESMODEL_H_*/
