@@ -28,8 +28,8 @@
 #define UPLOADCONTROLLER_H_
 
 #include <QPersistentModelIndex>
-#include <QStandardItemModel>
 
+#include "gnunet_qt_common.h"
 #include "fs.h"
 #include "uploadDialog.h"
 #include "uploadItemDelegate.h"
@@ -37,28 +37,35 @@
 class GFSUploadController : public QObject
 {
   Q_OBJECT
-  
+
 public:
+  typedef enum {COL_FILENAME, COL_PROGRESS, COL_STATUS, COL_URI, COL_COUNT} GFSUploadControllerCols;
+
 	GFSUploadController(GFSPlugin *fs);
 	virtual ~GFSUploadController();
-  
+
   bool start(QWidget *parent, const QString &strPath, bool index,
     int prio, int anon);
   QPersistentModelIndex *started(QPersistentModelIndex *parent,
-    QString path, unsigned long long total, unsigned long long completed);
+    const GNUNET_FSUI_Event *event);
+  QPersistentModelIndex *resumed(QPersistentModelIndex *parent,
+      const GNUNET_FSUI_Event *event);
   void progress(QPersistentModelIndex *idx, unsigned long long completed,
     unsigned long long total);
-  void state(QPersistentModelIndex *idx, GNUNET_FSUI_EventType event);
+  void complete(QPersistentModelIndex *idx, GFSEcrsUri uri);
+  void state(QPersistentModelIndex *idx, GNUNET_FSUI_EventType type);
   void clear();
-  
+
   QAbstractItemModel *model();
 protected:
   void setProgress(QPersistentModelIndex *idx, unsigned long long completed,
     unsigned long long total);
+  QPersistentModelIndex * newUpload(QPersistentModelIndex *parent, const char *fn,
+      unsigned long long total, unsigned long long complete);
 
   GFSPlugin *fs;
   EXTRACTOR_ExtractorList *extractors;
-  QStandardItemModel uploadModel;
+  GItemModel uploadModel;
   GFSUploadItemDelegate delegate;
 };
 
