@@ -42,12 +42,17 @@ public:
   GGeneralPlugin(struct GNUNET_GC_Configuration *config,
     struct GNUNET_GE_Context *errorContext, QObject *parent = NULL);
   virtual ~GGeneralPlugin();
+
+signals:
+  int setStatusText(const QString &strIcon, const QString &strText);
+  int setNetworkStatus(const QString &strIcon, const QString &strText);
+
+
 protected:
   void updateUi();
-  
+
   GStartStopThread *startStopThread;
   GCheckDaemonThread *checkDaemonThread;
-  QTimer timer;
   int runs;
   bool isRunning, pending;
 protected slots:
