@@ -30,6 +30,10 @@
 
 #include "startStopThread.h"
 
+#if defined(Q_OS_WIN)
+  extern "C" char *_win_strerror(int errnum);
+#endif
+
 GStartStopThread::GStartStopThread(struct GNUNET_GC_Configuration *config,
     struct GNUNET_GE_Context *errorContext, QObject *parent) : QThread(parent)
 {
@@ -51,18 +55,24 @@ void GStartStopThread::start(bool doStart)
 void GStartStopThread::run()
 {
   bool ret;
-  
+
   if (doStart)
     ret = GNUNET_daemon_start(errorContext, config, NULL, GNUNET_YES) != GNUNET_SYSERR;
   else
   {
     struct GNUNET_ClientServerConnection * sock;
-    
+
     sock = GNUNET_client_connection_create(errorContext, config);
     ret = GNUNET_client_connection_request_daemon_shutdown(sock) != GNUNET_SYSERR;
   }
-    
-  emit finished(ret, GNUNET_GE_strerror(errno));
+
+  emit finished(ret,
+#if defined(Q_OS_WIN)
+      _win_strerror(errno)
+#else
+      strerror(errno)
+#endif
+  );
 }
 
 /* end of startStopThread.cc */
