@@ -29,7 +29,7 @@
 
 #include <QThread>
 
-class GStartStopThread : protected QThread
+class GStartStopThread : public QThread
 {
   Q_OBJECT
 public:
