@@ -35,7 +35,7 @@ GStatsThread::GStatsThread(struct GNUNET_GC_Configuration *config,
 {
   this->config = config;
   this->errorContext = errorContext;
-  
+
   stopSignalled = false;
 }
 
@@ -46,15 +46,15 @@ static int acquireStatistics(const char *name, unsigned long long value, void *p
 
 bool GStatsThread::processStat(const char *name, unsigned long long value)
 {
-   emit stat(name, value);
-   
+   emit stat(QString::fromUtf8(name), value);
+
    return !stopSignalled;
 }
 
 void GStatsThread::run()
 {
   int res;
-  QTime timer;
+  GNUNET_CronTime end;
   struct GNUNET_ClientServerConnection *sock;
 
   sock = GNUNET_client_connection_create(errorContext, config);
@@ -67,19 +67,19 @@ void GStatsThread::run()
 
   while (! stopSignalled)
   {
-    timer.start();
+    end = GNUNET_get_time() + 1 * GNUNET_CRON_SECONDS;
     res = GNUNET_STATS_get_statistics(errorContext,
             sock,
-            &acquireStatistics, 
+            &acquireStatistics,
             this);
     if (res != GNUNET_OK)
       GNUNET_GE_LOG(errorContext, (GNUNET_GE_KIND) (GNUNET_GE_ERROR | GNUNET_GE_USER | GNUNET_GE_IMMEDIATE),
         qPrintable(tr("Error reading information from background process gnunetd.")));
 
-    while (timer.elapsed() < 1000 && !stopSignalled)
+    while (GNUNET_get_time() < end && !stopSignalled)
       msleep(100);
   }
-  
+
   GNUNET_client_connection_destroy(sock);
 }
 
